﻿using System;
using System.IO;
using Xunit;
using Newtonsoft.Json;

namespace Field
{
    [Collection("Serial")]
    public class TestReports
    {
        private Reports reports;

        static TestReports()
        {
            try
            {
                Directory.Delete("out", true);
            }
            catch { }
            Directory.CreateDirectory("out");
        }

        public TestReports()
        {
            this.reports = new Field.Reports();
        }

        [Fact]
        public void バージョンが取得できる() 
        {
            var version = reports.Version();
            Assert.NotNull(version);
        }

        [Fact]
        public void PDF文字列を生成できる()
        {
            var param = new {
                template = new {
                    paper = "A4"
                },
                context = new {
                    hello = new {
                        @new = "Tx",
                        value = "Hello, World!",
                        rect = new int[] {100, 700, 400, 750}
                    }
                }
            };
            var pdf = reports.Renders(JsonConvert.SerializeObject(param));
            Assert.IsType<byte[]>(pdf);
            // Console.WriteLine("Renders: {0}", System.Text.Encoding.ASCII.GetString(pdf));
        }

        [Fact]
        public void PDFファイルを生成できる()
        {
            reports.Render("{}", "out/empty.pdf");
            Assert.True(File.Exists("out/empty.pdf"));
        }

        [Fact]
        public void パースエラーで例外が発生する()
        {
            Assert.Throws<Exception>(() => {
                var pdf = reports.Renders("{");
            });
        }

        [Fact]
        public void 動的に組み立てたパラメータでPDFを生成できる()
        {
            var param = new {
                template = new {
                    paper = "A4"
                },
                context = new {
                    hello = new {
                        @new = "Tx",
                        value = "Hello, World!",
                        rect = new int[] {100, 700, 400, 750}
                    }
                }
            };
            reports.Render(JsonConvert.SerializeObject(param), "out/hello.pdf");
            Assert.True(File.Exists("out/hello.pdf"));
        }
    }
}
