﻿using System;
using System.Text;
using System.Runtime.InteropServices;

namespace Field {
#if COM_VISIBLE
    [ComVisible(true)]
    [Guid("E1414E51-E9C9-3161-8254-99B8BA70F9CE")]
    [InterfaceType(ComInterfaceType.InterfaceIsDual)]
#endif
    public interface IReports
    {
        void SetLogLevel(int n);

        void SetLogFlag(int f);

        /// <summary>
        /// バージョン番号を取得します。
        /// </summary>
        /// <returns>バージョン番号</returns>
        string Version();

        /// <summary>
        /// レンダリング・パラメータのデフォルト値を設定します。
        /// </summary>
        /// <param name="param">JSON文字列形式レンダリング・パラメータ</param>
        void SetDefaults(string param);

        /// <summary>
        /// レンダリング・パラメータを元にレンダリングを実行し，結果をバイト文字列として返します。
        /// </summary>
        /// <param name="param">JSON文字列形式レンダリング・パラメータ</param>
        /// <returns>PDFデータ</returns>
        byte[] Renders(string param);

        /// <summary>
        /// レンダリング・パラメータを元にレンダリングを実行します。
        /// 処理結果は，指定したファイルに出力されます。
        /// </summary>
        /// <param name="param">JSON文字列形式レンダリング・パラメータ</param>
        /// <param name="filename">出力先ファイル名</param>
        void Render(string param, string filename);

        /// <summary>
        /// レンダリング・パラメータを元に電子署名を実行し，結果をバイト文字列として返します。
        /// </summary>
        /// <param name="param">JSON文字列形式レンダリング・パラメータ</param>
        /// <returns>PDFデータ</returns>
        byte[] Signs(string param);

        /// <summary>
        /// レンダリング・パラメータを元に電子署名を実行します。
        /// 処理結果は，指定したファイルに出力されます。
        /// </summary>
        /// <param name="param">JSON文字列形式レンダリング・パラメータ</param>
        /// <param name="filename">出力先ファイル名</param>
        void Sign(string param, string filename);

        /// <summary>
        /// PDF署名を検証し，検証結果とPDFを返します。
        /// </summary>
        /// <param name="param">JSON文字列形式レンダリング・パラメータ</param>
        Tuple<string, byte[]> Validates(string param);

        /// <summary>
        /// PDF署名を検証し，検証結果を文字列として返します。
        /// 同時に，検証情報を埋め込んだPDFを指定したファイルに出力します。
        /// </summary>
        /// <param name="param">JSON文字列形式レンダリング・パラメータ</param>
        /// <param name="filename">出力先ファイル名</param>
        string Validate(string param, string filename);
    }
}