#include <jni.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
extern "C" {
#include "jp_co_field_works_Reports.h"
#include "reports.h"
}

static void throw_exn(JNIEnv* env, caml_value* exn)
{
    char msg[RP_MAX_ERR_MSG_LEN], buf[RP_MAX_ERR_MSG_LEN+20];
    int err;
    const char* tag = rp_get_error(exn, &err, msg);
    snprintf(buf, sizeof(buf), "%s: %s", tag, msg);
    jclass clazz = env->FindClass("jp/co/field_works/ReportsException");
    env->ThrowNew(clazz, buf);
}

static jstring get_jstring(JNIEnv* env, caml_value* v)
{
    jstring jresult = NULL;
    if (!rp_is_error(v)) {
        int len = rp_get_string_val(v, NULL);
        char* buf = (char*)malloc(len + 1);
        if (buf) {
            rp_get_string_val(v, buf);
            buf[len] = '\0';
            jresult = env->NewStringUTF(buf);
            free(buf);
        }
    }
    return jresult;
}

static jbyteArray get_jbyteArray(JNIEnv* env, caml_value* v)
{
    jbyteArray jresult = NULL;
    int len = rp_get_string_val(v, NULL);
    char* buf = (char*)malloc(len);
    if (buf) {
        rp_get_string_val(v, buf);
        jresult = env->NewByteArray(len);
        env->SetByteArrayRegion(jresult, 0, len, (jbyte*)buf);
        free(buf);
    }
    return jresult;
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL
Java_jp_co_field_1works_Reports_init(JNIEnv* env, jclass clazz)
{
    rp_init();
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    version
 * Signature: ()Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL
Java_jp_co_field_1works_Reports_version(JNIEnv* env, jclass clazz)
{
    jstring jresult = NULL;
    caml_value* caml_result = rp_version();
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
        goto exit;
    }
    jresult = get_jstring(env, caml_result);

exit:
    if (caml_result) rp_free(caml_result);
    return jresult;
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    setLogLevel
 * Signature: (I)V
 */
JNIEXPORT void JNICALL
Java_jp_co_field_1works_Reports_setLogLevel(JNIEnv* env, jclass clazz, jint n)
{
    caml_value* caml_result = rp_set_log_level(n);
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
    }
    rp_free(caml_result);
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    setDefaults
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL
Java_jp_co_field_1works_Reports_setDefaults(JNIEnv* env, jclass clazz, jstring jparam)
{
    const char* param = NULL;
    caml_value* caml_param = NULL;
    caml_value* caml_result = NULL;

    param = env->GetStringUTFChars(jparam, NULL);
    caml_param = rp_json_of_string(param);
    if (rp_is_error(caml_param)) {
        throw_exn(env, caml_param);
        goto exit;
    }
    caml_result = rp_set_defaults(caml_param);
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
        goto exit;
    }

exit:
    if (param) env->ReleaseStringUTFChars(jparam, param);
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    renders
 * Signature: (Ljava/lang/String;)[B
 */
JNIEXPORT jbyteArray JNICALL
Java_jp_co_field_1works_Reports_renders(JNIEnv* env, jclass clazz, jstring jparam)
{
    const char* param = NULL;
    caml_value* caml_param = NULL;
    caml_value* caml_result = NULL;
    jbyteArray jresult = NULL;

    param = env->GetStringUTFChars(jparam, NULL);
    caml_param = rp_json_of_string(param);
    if (rp_is_error(caml_param)) {
        throw_exn(env, caml_param);
        goto exit;
    }
    caml_result = rp_pdf_of_json(caml_param);
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
        goto exit;
    }
    jresult = get_jbyteArray(env, caml_result);

exit:
    if (param) env->ReleaseStringUTFChars(jparam, param);
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
    return jresult;
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    render
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL
Java_jp_co_field_1works_Reports_render(JNIEnv* env, jclass clazz, jstring jparam, jstring jfname)
{
    const char* param = NULL;
    const char* fname = NULL;
    caml_value* caml_param = NULL;
    caml_value* caml_result = NULL;

    param = env->GetStringUTFChars(jparam, NULL);
    fname = env->GetStringUTFChars(jfname, NULL);
    caml_param = rp_json_of_string(param);
    if (rp_is_error(caml_param)) {
        throw_exn(env, caml_param);
        goto exit;
    }

    caml_result = rp_write_pdf_from_json(caml_param, fname);
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
        goto exit;
    }

exit:
    if (param) env->ReleaseStringUTFChars(jparam, param);
    if (fname) env->ReleaseStringUTFChars(jfname, fname);
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    signs
 * Signature: (Ljava/lang/String;)[B
 */
JNIEXPORT jbyteArray JNICALL Java_jp_co_field_1works_Reports_signs(JNIEnv *env, jclass clazz, jstring jparam)
{
    const char* param = NULL;
    caml_value* caml_param = NULL;
    caml_value* caml_result = NULL;
    jbyteArray jresult = NULL;

    param = env->GetStringUTFChars(jparam, NULL);
    caml_param = rp_json_of_string(param);
    if (rp_is_error(caml_param)) {
        throw_exn(env, caml_param);
        goto exit;
    }
    caml_result = rp_signedpdf_of_json(caml_param);
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
        goto exit;
    }
    jresult = get_jbyteArray(env, caml_result);

exit:
    if (param) env->ReleaseStringUTFChars(jparam, param);
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
    return jresult;
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    sign
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_jp_co_field_1works_Reports_sign(JNIEnv *env, jclass clazz, jstring jparam, jstring jfname)
{
    const char* param = NULL;
    const char* fname = NULL;
    caml_value* caml_param = NULL;
    caml_value* caml_result = NULL;

    param = env->GetStringUTFChars(jparam, NULL);
    fname = env->GetStringUTFChars(jfname, NULL);
    caml_param = rp_json_of_string(param);
    if (rp_is_error(caml_param)) {
        throw_exn(env, caml_param);
        goto exit;
    }

    caml_result = rp_write_signedpdf_from_json(caml_param, fname);
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
        goto exit;
    }

exit:
    if (param) env->ReleaseStringUTFChars(jparam, param);
    if (fname) env->ReleaseStringUTFChars(jfname, fname);
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    validates
 * Signature: (Ljava/lang/String;)Ljp/co/field_works/ValidateResult;
 */
JNIEXPORT jobject JNICALL Java_jp_co_field_1works_Reports_validates
(JNIEnv *env, jclass clazz, jstring jparam)
{
    const char* param = NULL;
    caml_value* caml_param = NULL;
    caml_value* caml_result = NULL;
    jobject jresult = NULL;
    jclass jcls = env->FindClass("jp/co/field_works/ValidateResult");
    jmethodID cns = env->GetMethodID(jcls, "<init>", "(Ljava/lang/String;[B)V");

    param = env->GetStringUTFChars(jparam, NULL);
    caml_param = rp_json_of_string(param);
    if (rp_is_error(caml_param)) {
        throw_exn(env, caml_param);
        goto exit;
    }
    caml_result = rp_validatedpdf_of_json(caml_param);
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
        goto exit;
    }
    jresult = env->NewObject(jcls, cns,
        get_jstring(env, rp_get_field(caml_result, 0)),
        get_jbyteArray(env, rp_get_field(caml_result, 1)));

exit:
    if (param) env->ReleaseStringUTFChars(jparam, param);
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
    return jresult;
}

/*
 * Class:     jp_co_field_works_Reports
 * Method:    validate
 * Signature: (Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_jp_co_field_1works_Reports_validate(JNIEnv *env, jclass clazz, jstring jparam, jstring jfname)
{
    const char* param = NULL;
    const char* fname = NULL;
    caml_value* caml_param = NULL;
    caml_value* caml_result = NULL;
    jstring jresult = NULL;

    param = env->GetStringUTFChars(jparam, NULL);
    fname = env->GetStringUTFChars(jfname, NULL);
    caml_param = rp_json_of_string(param);
    if (rp_is_error(caml_param)) {
        throw_exn(env, caml_param);
        goto exit;
    }

    caml_result = rp_write_validatedpdf_from_json(caml_param, fname);
    if (rp_is_error(caml_result)) {
        throw_exn(env, caml_result);
        goto exit;
    }
    jresult = get_jstring(env, caml_result);

exit:
    if (param) env->ReleaseStringUTFChars(jparam, param);
    if (fname) env->ReleaseStringUTFChars(jfname, fname);
    if (caml_param) rp_free(caml_param);
    if (caml_result) rp_free(caml_result);
    return jresult;
}
